﻿using BeSafePlus.Database;
using BeSafePlus.Models;
using BeSafePlus.Sensors;
using BeSafePlus.Services.Interfaces;
using System.Diagnostics;
using System.Net.Http.Headers;
using System.Text;
using System.Text.Json;

namespace BeSafePlus.Services
{
    public class DailyReportService : IReportService
    {
        private readonly ILocalDatabase _localDatabase;
        private readonly HttpClient _httpClient;
        private JsonSerializerOptions SerializerOptions => new()
        {
            PropertyNamingPolicy = JsonNamingPolicy.CamelCase,
            WriteIndented = true,
        };

        private double _activeHours;
        private double _stationaryHours;
        private double _phoneInUseHours;
        private readonly DateTime EndOfDay = DateTime.Today.AddDays(1); // this is for testing, update with real dates as report will be generated after midnight
        private readonly DateTime StartOfDay = DateTime.Today; // this is for testing, update with real dates as report will be generated after midnight
        public DailyReportService(ILocalDatabase localDatabase)
        {
            _httpClient = new HttpClient();
            _localDatabase = localDatabase;
        }

        public async Task WriteAndSendReport()
        {
            try
            {
                //   await _localDatabase.SaveActivityLevel(SensorEventHandler.UserActivityState.Stationary, DateTime.Now.AddDays(-1).AddHours(4)); // for testing

                var dailyAcitvityList = (await _localDatabase.GetTodaysActivity()).OrderBy(a => a.Time).ToList();
                var dailySteps = (await _localDatabase.GetStepCountByDate(DateTime.Now.AddDays(-1)))?.Steps;

                // Skip if too few data is available
                if (dailyAcitvityList == null || dailyAcitvityList.Count < 3)
                {
                    Console.WriteLine("nothing to report");
                    return;
                }

                var convertedList = ActivityLevelDto.ConvertToDtoList(dailyAcitvityList);

                var firstEntry = convertedList.First(); // this will be the last entry of the day before or first ever entry, if the app hasnt been used before
                var firstEntryOfDay = convertedList.Skip(1).First(); // first entry of the day of the report
                var lastEntry = convertedList.Last(); // last entry of the day which will be firstEntry of next report

                // Get the last recorded activity state before report day to get which state should be attributed hours from midnight to firstEntryOfDay
                var lastActivityStateBeforeDayStart = firstEntry.ActivityState;

                // If firstEntry is not from day before, the app is missing data, which will happen on first day of download.
                if (firstEntry.Time < StartOfDay)
                {
                    // Get time from midnight to first recorded event of the day
                    var hoursFromMidnightToFirstEntry = (firstEntryOfDay.Time - StartOfDay).TotalHours;

                    AddHours(lastActivityStateBeforeDayStart, hoursFromMidnightToFirstEntry);
                }
                else
                {
                    var firstAvailableHours = (firstEntryOfDay.Time - firstEntry.Time).TotalHours;
                    AddHours(lastActivityStateBeforeDayStart, firstAvailableHours);
                }

                var lastEntryPtr = firstEntryOfDay;

                // Skip the first 2 recordings that has already been handled
                foreach (var nextEntryPtr in convertedList.Skip(2))
                {
                    var time = (nextEntryPtr.Time - lastEntryPtr.Time).TotalHours;
                    AddHours(lastEntryPtr.ActivityState, time);
                    if (nextEntryPtr == convertedList.Last())
                    {
                        AddHoursRelativeToEndOfDay(nextEntryPtr);
                    }
                    lastEntryPtr = nextEntryPtr;
                }

                var total = _activeHours + _stationaryHours + _phoneInUseHours;

                if (total > 24)
                {
                    Debug.WriteLine("error in hour calculations, total hours more than 24");
                    return;
                }
                // either hardcode percentage based on 24 or total recorded hours

                //var activePercentage = (activeHours / total) * 100;
                //var notUsedPercentage = (notusedHours / total) * 100;
                //var phoneInUsePercentage = (phoneHours / total) * 100;
                var activePercentage = (_activeHours / 24) * 100;
                var notUsedPercentage = (_stationaryHours / 24) * 100;
                var phoneInUsePercentage = (_phoneInUseHours / 24) * 100;

                var activityReport = new ActivityReport
                {
                    ActiveHours = _activeHours,
                    PhoneInUseHours = _phoneInUseHours,
                    StationaryHours = _stationaryHours,
                    ActivePercentage = activePercentage,
                    PhoneInUsePercentage = phoneInUsePercentage,
                    StationaryPercentage = notUsedPercentage,
                };

                var steps = new Steps
                {
                    TotalSteps = dailySteps ?? 0,
                    HourlySteps = [] // fill or remove
                };


                Debug.WriteLine(activityReport);

                // perform http request to REDCap
                await PostReport(activityReport, steps);
                // handle failure?

                // Flush all activity level entries after calculations to save memory 
             //   await _localDatabase.FlushActivityLevel();

                // Write the last entry of report day, to be used for calculation in next report
                //await _localDatabase.SaveActivityLevel(lastEntry.ActivityState, lastEntry.Time);
            }
            catch (Exception ex)
            {
                Console.WriteLine($"report generation failed: {ex.Message}");
                return;
            }
        }
        // maybe move to its own client class
        private async Task PostReport(ActivityReport activityReport, Steps steps)
        {
            if (activityReport == null || steps == null)
            {
                return;
            }

            // get endpoint and token from localdatabase?
            var endpoint = "https://besafeplus.onrender.com/verify";

            var redCapEntry = new RedCapEntry
            {
                ActivityReport = activityReport,
                Steps = steps
            };

            try
            {
                var report = JsonSerializer.Serialize<RedCapEntry>(redCapEntry, SerializerOptions);
                var content = new StringContent(report, Encoding.UTF8, "application/json");

                var request = new HttpRequestMessage(HttpMethod.Post, endpoint)
                {
                    Content = content
                };

                var response = await _httpClient.SendAsync(request);

                if (response.IsSuccessStatusCode)
                {
                    Debug.WriteLine("All good, report was sent");
                }
                else
                {
                    Debug.WriteLine($"{response.StatusCode}");
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine($"Error in {nameof(DailyReportService)}: {ex.Message}");
            }
        }
        private void AddHours(SensorEventHandler.UserActivityState userActivityState, double time)
        {
            switch (userActivityState)
            {
                case SensorEventHandler.UserActivityState.Active:
                    _activeHours += time;
                    break;
                case SensorEventHandler.UserActivityState.Stationary:
                    _stationaryHours += time;
                    break;
                case SensorEventHandler.UserActivityState.PhoneInUse:
                    _phoneInUseHours += time;
                    break;
            }
        }
        private void AddHoursRelativeToEndOfDay(ActivityLevelDto activityLevel)
        {
            var time = (EndOfDay - activityLevel.Time).TotalHours;
            AddHours(activityLevel.ActivityState, time);
        }

        class RedCapEntry
        {
            public ActivityReport? ActivityReport { get; set; }

            public Steps? Steps { get; set; }

        }
        class Steps
        {
            public long TotalSteps { get; set; } = 0;
            public List<HourlySteps> HourlySteps { get; set; } = [];

        }
        class HourlySteps
        {
            public int Steps { get; set; }
            public DateTime Hour { get; set; }
        }
        class ActivityReport
        {
            public double ActivePercentage { get; set; }
            public double PhoneInUsePercentage { get; set; }
            public double StationaryPercentage { get; set; }
            public double ActiveHours { get; set; }
            public double PhoneInUseHours { get; set; }
            public double StationaryHours { get; set; }

        }



    }
}
