﻿using CommunityToolkit.Mvvm.ComponentModel;
using CommunityToolkit.Mvvm.Input;
using System;
using System.Collections.ObjectModel;
using BeSafePlus.Models;
using System.Threading.Tasks;
using BeSafePlus.Database;
using System.Text.RegularExpressions;
using System.Globalization;
using CommunityToolkit.Mvvm.Messaging;
using CommunityToolkit.Mvvm.Messaging.Messages;
using BeSafePlus.Views;
using System.Diagnostics;
using BeSafePlus.Services;

namespace BeSafePlus.ViewModels
{
    public partial class SettingsViewModel : ObservableObject
    {
        private readonly ILocalDatabase _database;

        private readonly IUIService _UIService;

        private bool _isActivityHistoryEnabled;

        private bool _isActivityHistoryDisabled;

        private bool _isCircleEnabled;

        private bool _isCircleDisabled;

        private bool _isProgressEnabled;

        private bool _isProgressDisabled;

        private bool _isDistanceEnabled;

        private bool _isDistanceDisabled;

        private bool _isDailyGoalEnabled;

        private bool _isDailyGoalDisabled;

        private bool _isCaloriesEnabled;

        private bool _isCaloriesDisabled;

        private string _newStepGoal;

        private bool _isProfilePopupVisible;

        private bool _isGoalPopupVisible;

        private bool _isTokenScanNeeded;
        public SettingsViewModel(ILocalDatabase database, IUIService uIService)
        {
            _database = database;

            IsActivityHistoryEnabled = Preferences.Get("IsActivityHistoryEnabled", false);

            IsCircleEnabled = Preferences.Get("IsCircleEnabled", false);

            IsProgressEnabled = Preferences.Get("IsProgressEnabled", false);

            IsDistanceEnabled = Preferences.Get("IsDistanceEnabled", false);

            IsDailyGoalEnabled = Preferences.Get("IsDailyGoalEnabled", false);

            IsCaloriesEnabled = Preferences.Get("IsCaloriesEnabled", false);

            if (!IsDailyGoalEnabled)
            {
                IsProgressEnabled = false;
            }

            LoadCurrentGoal();

            TokenAvailabilityCheck();

            SetThemeToggle();

            GenderOptions = new ObservableCollection<string> { "Mand", "Kvinde", "Andet" };

            _UIService = uIService;
        }
        public string NewStepGoal
        {
            get => _newStepGoal;
            set => SetProperty(ref _newStepGoal, value);
        }
        public bool IsProfilePopupVisible
        {
            get => _isProfilePopupVisible;
            set => SetProperty(ref _isProfilePopupVisible, value);
        }
        public bool IsGoalPopupVisible
        {
            get => _isGoalPopupVisible;
            set => SetProperty(ref _isGoalPopupVisible, value);
        }
        private async void LoadCurrentGoal()
        {
            var stepCount = await _database.GetStepCountByDate(System.DateTime.Now);

            if (stepCount != null)
            {
                NewStepGoal = stepCount.StepGoal?.ToString() ?? "5000"; ;
            }
        }
        public bool IsActivityHistoryEnabled
        {
            get => _isActivityHistoryEnabled;
            set
            {
                SetProperty(ref _isActivityHistoryEnabled, value);

                WeakReferenceMessenger.Default.Send(new ActivityHistoryChangedMessage { IsEnabled = value });

                Preferences.Set("IsActivityHistoryEnabled", value);
            }
        }
        public bool IsActivityHistoryDisabled
        {
            get => _isActivityHistoryDisabled;
            set => SetProperty(ref _isActivityHistoryDisabled, value);
        }
        public bool IsCircleEnabled
        {
            get => _isCircleEnabled;
            set
            {
                SetProperty(ref _isCircleEnabled, value);

                WeakReferenceMessenger.Default.Send(new CircleVisibilityChangedMessage { IsEnabled = value });

                Preferences.Set("IsCircleEnabled", value);
            }
        }
        public bool IsCircleDisabled
        {
            get => _isCircleDisabled;
            set => SetProperty(ref _isCircleDisabled, value);
        }
        public bool IsProgressEnabled
        {
            get => _isProgressEnabled;
            set
            {
                SetProperty(ref _isProgressEnabled, value);

                WeakReferenceMessenger.Default.Send(new ProgressChangedMessage(value));

                Preferences.Set("IsProgressEnabled", value);
            }
        }
        public bool IsProgressDisabled
        {
            get => _isProgressDisabled;
            set => SetProperty(ref _isProgressDisabled, value);
        }
        public bool IsDistanceEnabled
        {
            get => _isDistanceEnabled;
            set
            {
                SetProperty(ref _isDistanceEnabled, value);

                WeakReferenceMessenger.Default.Send(new DistanceChangedMessage { IsEnabled = value });

                Preferences.Set("IsDistanceEnabled", value);
            }
        }
        public bool IsDistanceDisabled
        {
            get => _isDistanceDisabled;
            set => SetProperty(ref _isDistanceDisabled, value);
        }
        public bool IsDailyGoalEnabled
        {
            get => _isDailyGoalEnabled;
            set
            {
                SetProperty(ref _isDailyGoalEnabled, value);

                WeakReferenceMessenger.Default.Send(new DailyGoalVisibilityChangedMessage { IsEnabled = value });

                if(!value)
                {
                    // Disable goal related stuff if goal is disabled
                    IsProgressEnabled = false;
                    IsCircleEnabled = false;
                }

                Preferences.Set("IsDailyGoalEnabled", value);
            }
        }
        public bool IsDailyGoalDisabled
        {
            get => _isDailyGoalDisabled;
            set => SetProperty(ref _isDailyGoalDisabled, value);
        }
        public bool IsCaloriesEnabled
        {
            get => _isCaloriesEnabled;
            set
            {
                SetProperty(ref _isCaloriesEnabled, value);

                WeakReferenceMessenger.Default.Send(new CaloriesVisibilityChangedMessage { IsEnabled = value });

                Preferences.Set("IsCaloriesEnabled", value);
            }
        }
        public bool IsCaloriesDisabled
        {
            get => _isCaloriesDisabled;
            set => SetProperty(ref _isCaloriesDisabled, value);
        }
        public bool IsTokenScanNeeded
        {
            get => _isTokenScanNeeded;
            set => SetProperty(ref _isTokenScanNeeded, value);
        }
        // maybe not have this and check in scan instead (for token validity, so user doesnt remove token on accident) in case token needs to be updated
        public async void TokenAvailabilityCheck()
        {
            var token = await _database.GetRedCapToken();

            if (token != null)
            {
                IsTokenScanNeeded = false;
            }

            else
            {
                IsTokenScanNeeded = true;
            }
        }

        [ObservableProperty]
        private bool isDarkMode;
        partial void OnIsDarkModeChanged(bool value)
        {
            Preferences.Default.Set("DarkMode", value);

            if (Application.Current != null)
            {
                Application.Current.UserAppTheme = value ? AppTheme.Dark : AppTheme.Light;
                
                _UIService?.ApplyTheme(value);
            }

            else
            {
                Debug.WriteLine("Warning: Application.Current is null");
            }
        }
        private void SetThemeToggle()
        {
            if (Preferences.Default.ContainsKey("DarkMode"))
            {
                IsDarkMode = Preferences.Default.Get("DarkMode", true);
            }

            else
            {
                IsDarkMode = Application.Current?.RequestedTheme == AppTheme.Dark;
            }
        }

        [RelayCommand]
        public async void ScanToken()
        {
            await Shell.Current.GoToAsync(nameof(QrScannerPage));
        }

        [RelayCommand]
        public void EnableActivityHistory()
        {
            IsActivityHistoryEnabled = true;

            Preferences.Set("IsActivityHistoryEnabled", true);
        }

        [RelayCommand]
        public void DisableActivityHistory()
        {
            IsActivityHistoryEnabled = false;

            Preferences.Set("IsActivityHistoryEnabled", false);
        }

        [RelayCommand]
        public void EnableCircle()
        {
            IsCircleEnabled = true;

            Preferences.Set("IsCircleEnabled", true);
        }

        [RelayCommand]
        public void DisableCircle()
        {
            IsCircleEnabled = false;

            Preferences.Set("IsCircleEnabled", false);
        }

        [RelayCommand]
        public void EnableProgress()
        {
            IsProgressEnabled = true;

            Preferences.Set("IsProgressEnabled", true);
        }

        [RelayCommand]
        public void DisableProgress()
        {
            IsProgressEnabled = false;

            Preferences.Set("IsProgressEnabled", false);
        }

        [RelayCommand]
        public void EnableDistance()
        {
            IsDistanceEnabled = true;

            Preferences.Set("IsDistanceEnabled", true);
        }

        [RelayCommand]
        public void DisableDistance()
        {
            IsDistanceEnabled = false;

            Preferences.Set("IsDistanceEnabled", false);
        }

        [RelayCommand]
        public void EnableDailyGoal()
        {
            IsDailyGoalEnabled = true;

            Preferences.Set("IsDailyGoalEnabled", true);
        }

        [RelayCommand]
        public void DisableDailyGoal()
        {
            IsDailyGoalEnabled = false;

            Preferences.Set("IsDailyGoalEnabled", false);
        }

        [RelayCommand]
        public void EnableCalories()
        {
            IsCaloriesEnabled = true;

            Preferences.Set("IsCaloriesEnabled", true);
        }

        [RelayCommand]
        public void DisableCalories()
        {
            IsCaloriesEnabled = false;

            Preferences.Set("IsCaloriesEnabled", false);
        }

        [RelayCommand]
        public void CloseProfilePopup()
        {
            IsProfilePopupVisible = false;
        }

        [RelayCommand]
        public void OpenEditProfilePopup()
        {
            IsProfilePopupVisible = true;
        }

        [RelayCommand]
        public async Task SaveProfile()
        {
            if (string.IsNullOrWhiteSpace(NewProfileName) ||
                string.IsNullOrWhiteSpace(SelectedGender) ||
                string.IsNullOrWhiteSpace(NewProfileAge) ||
                string.IsNullOrWhiteSpace(NewProfileHeight) ||
                string.IsNullOrWhiteSpace(NewProfileWeight))
            {
                await Application.Current.MainPage.DisplayAlert("Fejl", "Alle felter skal være udfyldt", "OK");
                return;
            }

            if (!int.TryParse(NewProfileAge, out int age) ||
                !int.TryParse(NewProfileHeight, out int height) ||
                !IsValidWeight(NewProfileWeight, out string weight))
            {
                await Application.Current.MainPage.DisplayAlert("Fejl", "Alder, Højde og Vægt skal være numeriske værdier", "OK");

                return;
            }

            await _database.UpdateUserName(newProfileName);

            await _database.UpdateUserGender(SelectedGender);

            await _database.UpdateUserAge(age);

            await _database.UpdateUserHeight(height);

            await _database.UpdateUserWeight(weight);

            Preferences.Set("Weight", double.Parse(weight, CultureInfo.InvariantCulture));

            await Application.Current.MainPage.DisplayAlert("Succes", "Profiloplysninger er nu blevet opdateret", "OK");

            IsProfilePopupVisible = false;
        }

        [RelayCommand]
        public void CloseGoalPopup()
        {
            IsGoalPopupVisible = false;
        }

        [RelayCommand]
        public void OpenEditDailyGoalPopup()
        {
            IsGoalPopupVisible = true;
        }

        [RelayCommand]
        public async Task SaveStepGoal()
        {
            if (int.TryParse(NewStepGoal, out int parsedGoal) && parsedGoal > 0)
            {
                await _database.UpdateStepGoal(parsedGoal);

                await Application.Current.MainPage.DisplayAlert("Succes", "Daglig Målsætning er nu blevet opdateret", "OK");

                IsGoalPopupVisible = false;

                return;
            }

            await Application.Current.MainPage.DisplayAlert("Fejl", "Indtast venligst en gyldig numerisk værdi mellem 1 og 5 cifre", "OK");

            IsGoalPopupVisible = true;
        }
        partial void OnSelectedGenderChanged(string value)
        {
            StepLength = value == "Mand" ? 0.79 : value == "Kvinde" ? 0.66 : 0.7; // Hvis Andet køn er valgt - Skridtlængde er 70 cm

            Preferences.Set("StepLength", StepLength);
        }
        private bool IsValidWeight(string input, out string weight)
        {
            if (decimal.TryParse(input, NumberStyles.AllowDecimalPoint, CultureInfo.InvariantCulture, out decimal parsedWeight) ||
                decimal.TryParse(input, NumberStyles.AllowDecimalPoint, CultureInfo.GetCultureInfo("dk-format"), out parsedWeight))
            {
                weight = parsedWeight.ToString();
                
                return true;
            }

            else
            {
                weight = null;

                return false;
            }
        }

        [ObservableProperty]
        private string newProfileName;

        [ObservableProperty]
        private string selectedGender;

        [ObservableProperty]
        private double stepLength;
        public ObservableCollection<string> GenderOptions { get; }

        [ObservableProperty]
        private string newProfileAge;

        [ObservableProperty]
        private string newProfileHeight;

        [ObservableProperty]
        private string newProfileWeight;
    }
    public class ActivityHistoryChangedMessage
    {
        public bool IsEnabled { get; set; }
    }
    public class CircleVisibilityChangedMessage
    {
        public bool IsEnabled { get; set; }
    }
    public class ProgressChangedMessage : ValueChangedMessage<bool>
    {
        public ProgressChangedMessage(bool isEnabled) : base(isEnabled) { }
    }
    public class DistanceChangedMessage
    {
        public bool IsEnabled { get; set; }
    }
    public class DailyGoalVisibilityChangedMessage
    {
        public bool IsEnabled { get; set; }
    }
    public class CaloriesVisibilityChangedMessage
    {
        public bool IsEnabled { get; set; }
    }
}